@@ -5,12 +5,14 @@ module RuboCop
     # Common functionality for nodes that are a kind of method dispatch:
     # `send`, `csend`, `super`, `zsuper`, `yield`, `defined?`,
     # and (modern only): `index`, `indexasgn`, `lambda`
-    module MethodDispatchNode
+    module MethodDispatchNode # rubocop:disable Metrics/ModuleLength
       extend NodePattern::Macros
       include MethodIdentifierPredicates
 
       ARITHMETIC_OPERATORS = %i[+ - * / % **].freeze
+      private_constant :ARITHMETIC_OPERATORS
       SPECIAL_MODIFIERS = %w[private protected].freeze
+      private_constant :SPECIAL_MODIFIERS
 
       # The receiving node of the method dispatch.
       #
@@ -26,10 +28,21 @@ module RuboCop
         node_parts[1]
       end
 
-      # The `block` node associated with this method dispatch, if any.
+      # The source range for the method name or keyword that dispatches this call.
       #
-      # @return [BlockNode, nil] the `block` node associated with this method
-      #                          call or `nil`
+      # @return [Parser::Source::Range] the source range for the method name or keyword
+      def selector
+        if loc?(:keyword)
+          loc.keyword
+        else
+          loc.selector
+        end
+      end
+
+      # The `block`, `numblock`, or `itblock` node associated with this method dispatch, if any.
+      #
+      # @return [BlockNode, nil] the `block`, `numblock`, or `itblock` node associated with this
+      #                          method call or `nil`
       def block_node
         parent if block_literal?
       end
@@ -92,7 +105,7 @@ module RuboCop
       #
       # @return [Boolean] whether the dispatched method is a setter
       def setter_method?
-        loc.respond_to?(:operator) && loc.operator
+        loc?(:operator)
       end
       alias assignment? setter_method?
 
@@ -104,7 +117,7 @@ module RuboCop
       #
       # @return [Boolean] whether the method was called with a connecting dot
       def dot?
-        loc.respond_to?(:dot) && loc.dot && loc.dot.is?('.')
+        loc_is?(:dot, '.')
       end
 
       # Checks whether the dispatched method uses a double colon to connect the
@@ -112,7 +125,7 @@ module RuboCop
       #
       # @return [Boolean] whether the method was called with a connecting dot
       def double_colon?
-        loc.respond_to?(:dot) && loc.dot && loc.dot.is?('::')
+        loc_is?(:dot, '::')
       end
 
       # Checks whether the dispatched method uses a safe navigation operator to
@@ -120,7 +133,7 @@ module RuboCop
       #
       # @return [Boolean] whether the method was called with a connecting dot
       def safe_navigation?
-        loc.respond_to?(:dot) && loc.dot && loc.dot.is?('&.')
+        loc_is?(:dot, '&.')
       end
 
       # Checks whether the *explicit* receiver of this method dispatch is
@@ -145,14 +158,14 @@ module RuboCop
       #
       # @return [Boolean] whether the method is the implicit form of `#call`
       def implicit_call?
-        method?(:call) && !loc.selector
+        method?(:call) && !selector
       end
 
       # Whether this method dispatch has an explicit block.
       #
       # @return [Boolean] whether the dispatched method has a block
       def block_literal?
-        parent&.block_type? && eql?(parent.send_node)
+        parent&.any_block_type? && eql?(parent.send_node)
       end
 
       # Checks whether this node is an arithmetic operation
@@ -163,16 +176,33 @@ module RuboCop
         ARITHMETIC_OPERATORS.include?(method_name)
       end
 
-      # Checks if this node is part of a chain of `def` modifiers.
+      # Checks if this node is part of a chain of `def` or `defs` modifiers.
       #
       # @example
       #
       #   private def foo; end
       #
-      # @return [Boolean] whether the dispatched method is a `def` modifier
-      def def_modifier?
-        send_type? &&
-          [self, *each_descendant(:send)].any?(&:adjacent_def_modifier?)
+      # @return whether the `def|defs` node is a modifier or not.
+      # See also `def_modifier` that returns the node or `nil`
+      def def_modifier?(node = self)
+        !!def_modifier(node)
+      end
+
+      # Checks if this node is part of a chain of `def` or `defs` modifiers.
+      #
+      # @example
+      #
+      #   private def foo; end
+      #
+      # @return [Node | nil] returns the `def|defs` node this is a modifier for,
+      # or `nil` if it isn't a def modifier
+      def def_modifier(node = self)
+        arg = node.children[2]
+
+        return unless node.send_type? && node.receiver.nil? && arg.is_a?(::AST::Node)
+        return arg if arg.any_def_type?
+
+        def_modifier(arg)
       end
 
       # Checks whether this is a lambda. Some versions of parser parses
@@ -191,7 +221,7 @@ module RuboCop
       #
       # @return [Boolean] whether this method is a lambda literal
       def lambda_literal?
-        block_literal? && loc.expression && loc.expression.source == '->'
+        loc.expression.source == '->' && block_literal?
       end
 
       # Checks whether this is a unary operation.
@@ -202,9 +232,9 @@ module RuboCop
       #
       # @return [Boolean] whether this method is a unary operation
       def unary_operation?
-        return false unless loc.selector
+        return false unless selector
 
-        operator_method? && loc.expression.begin_pos == loc.selector.begin_pos
+        operator_method? && loc.expression.begin_pos == selector.begin_pos
       end
 
       # Checks whether this is a binary operation.
@@ -213,22 +243,23 @@ module RuboCop
       #
       #   foo + bar
       #
-      # @return [Bookean] whether this method is a binary operation
+      # @return [Boolean] whether this method is a binary operation
       def binary_operation?
-        return false unless loc.selector
+        return false unless selector
 
-        operator_method? && loc.expression.begin_pos != loc.selector.begin_pos
+        operator_method? && loc.expression.begin_pos != selector.begin_pos
       end
 
       private
 
+      # @!method in_macro_scope?(node = self)
       def_node_matcher :in_macro_scope?, <<~PATTERN
         {
           root?                                    # Either a root node,
           ^{                                       # or the parent is...
             sclass class module class_constructor? # a class-like node
             [ {                                    # or some "wrapper"
-                kwbegin begin block
+                kwbegin begin any_block
                 (if _condition <%0 _>)  # note: we're excluding the condition of `if` nodes
               }
               #in_macro_scope?                     # that is itself in a macro scope
@@ -237,16 +268,19 @@ module RuboCop
         }
       PATTERN
 
+      # @!method adjacent_def_modifier?(node = self)
       def_node_matcher :adjacent_def_modifier?, <<~PATTERN
-        (send nil? _ ({def defs} ...))
+        (send nil? _ (any_def ...))
       PATTERN
 
+      # @!method bare_access_modifier_declaration?(node = self)
       def_node_matcher :bare_access_modifier_declaration?, <<~PATTERN
         (send nil? {:public :protected :private :module_function})
       PATTERN
 
+      # @!method non_bare_access_modifier_declaration?(node = self)
       def_node_matcher :non_bare_access_modifier_declaration?, <<~PATTERN
-        (send nil? {:public :protected :private :module_function} _)
+        (send nil? {:public :protected :private :module_function} _+)
       PATTERN
     end
   end
