@@ -718,9 +718,6 @@ unless foo then bar; else baz; end
 #----
 unless foo; bar; else baz; end
 
-#----
-proc {_1 = nil}
-
 #----
 begin ensure end
 
@@ -905,24 +902,6 @@ foo, bar = 1, 2
 #----
 foo, bar, baz = 1, 2
 
-#----
-proc {_1 = nil}
-
-#----
-_2 = 1
-
-#----
-proc {|_3|}
-
-#----
-def x(_4) end
-
-#----
-def _5; end
-
-#----
-def self._6; end
-
 #----
 meth rescue bar
 
@@ -1124,27 +1103,9 @@ bar def foo; self.each do end end
 #----
 case foo; in "a": then true; end
 
-#----
-case foo; in "#{ 'a' }": then true; end
-
-#----
-case foo; in "#{ %q{a} }": then true; end
-
-#----
-case foo; in "#{ %Q{a} }": then true; end
-
 #----
 case foo; in "a": 1 then true; end
 
-#----
-case foo; in "#{ 'a' }": 1 then true; end
-
-#----
-case foo; in "#{ %q{a} }": 1 then true; end
-
-#----
-case foo; in "#{ %Q{a} }": 1 then true; end
-
 #----
 a&.b &&= 1
 
@@ -1187,6 +1148,18 @@ f{ |**baz, &b| }
 #----
 if foo...bar; end
 
+#----
+!(...bar)
+
+#----
+!(..bar)
+
+#----
+!(bar...)
+
+#----
+!(bar..)
+
 #----
 !(foo...bar)
 
@@ -1417,9 +1390,6 @@ f{ |a, b,| }
 #----
 p begin 1.times do 1 end end
 
-#----
-retry
-
 #----
 p <<~E
 E
