@@ -123,6 +123,32 @@ RSpec.describe RuboCop::AST::IfNode do
     end
   end
 
+  describe '#then?' do
+    context 'with `then` keyword' do
+      let(:source) do
+        <<~SOURCE
+          if foo? then
+            1
+          end
+        SOURCE
+      end
+
+      it { is_expected.to be_then }
+    end
+
+    context 'without `then` keyword' do
+      let(:source) do
+        <<~SOURCE
+          if foo?
+            1
+          end
+        SOURCE
+      end
+
+      it { is_expected.not_to be_then }
+    end
+  end
+
   describe '#elsif?' do
     context 'with an elsif statement' do
       let(:source) do
@@ -138,7 +164,7 @@ RSpec.describe RuboCop::AST::IfNode do
       it { expect(elsif_node).to be_elsif }
     end
 
-    context 'with an if statement comtaining an elsif' do
+    context 'with an if statement containing an elsif' do
       let(:source) do
         ['if foo?',
          '  1',
