@@ -1,16 +1,13 @@
 # frozen_string_literal: true
 
-require 'parser/current'
-
 RSpec.describe RuboCop::AST::NodePattern do
   include RuboCop::AST::Sexp
 
   def parse(code)
-    buffer = Parser::Source::Buffer.new('(string)', 1, source: code)
-    builder = RuboCop::AST::Builder.new
-    Parser::CurrentRuby.new(builder).parse(buffer)
+    RuboCop::AST::ProcessedSource.new(code, ruby_version, '(string)').ast
   end
 
+  let(:ruby_version) { 3.3 }
   let(:root_node) { parse(ruby) }
   let(:node) { root_node }
   let(:params) { [] }
@@ -18,22 +15,14 @@ RSpec.describe RuboCop::AST::NodePattern do
   let(:instance) { described_class.new(pattern) }
   let(:method_name) { :match }
   let(:result) do
-    if keyword_params.empty? # Avoid bug in Ruby < 2.6
-      instance.match(node, *params)
-    else
-      instance.match(node, *params, **keyword_params)
-    end
+    instance.match(node, *params, **keyword_params)
   end
 
   RSpec::Matchers.define :match_code do |code, *params, **keyword_params|
     match do |pattern|
       instance = pattern.is_a?(String) ? described_class.new(pattern) : pattern
       code = parse(code) if code.is_a?(String)
-      if keyword_params.empty? # Avoid bug in Ruby < 2.6
-        instance.public_send(method_name, code, *params)
-      else
-        instance.public_send(method_name, code, *params, **keyword_params)
-      end
+      instance.public_send(method_name, code, *params, **keyword_params)
     end
   end
 
@@ -56,7 +45,7 @@ RSpec.describe RuboCop::AST::NodePattern do
   shared_examples 'invalid' do
     it 'is invalid' do
       expect { instance }
-        .to raise_error(RuboCop::AST::NodePattern::Invalid)
+        .to raise_error(described_class::Invalid)
     end
   end
 
@@ -145,15 +134,13 @@ RSpec.describe RuboCop::AST::NodePattern do
       it { expect(pattern).to match_code(node) }
     end
 
-    if RUBY_VERSION >= '2.6'
-      describe 'yaml compatibility' do
-        let(:instance) do
-          YAML.safe_load(YAML.dump(super()), permitted_classes: [described_class])
-        end
-        let(:ruby) { 'obj.method' }
-
-        it { expect(pattern).to match_code(node) }
+    describe 'yaml compatibility' do
+      let(:instance) do
+        YAML.safe_load(YAML.dump(super()), permitted_classes: [described_class])
       end
+      let(:ruby) { 'obj.method' }
+
+      it { expect(pattern).to match_code(node) }
     end
 
     describe '#==' do
@@ -245,6 +232,13 @@ RSpec.describe RuboCop::AST::NodePattern do
       it { expect(pattern).to match_code(node) }
     end
 
+    context 'empty string literals' do
+      let(:pattern) { '(str "")' }
+      let(:ruby) { '""' }
+
+      it { expect(pattern).to match_code(node) }
+    end
+
     context 'symbol literals' do
       let(:pattern) { '(sym :foo)' }
       let(:ruby) { ':foo' }
@@ -958,6 +952,168 @@ RSpec.describe RuboCop::AST::NodePattern do
 
       it_behaves_like 'invalid'
     end
+
+    context 'using repetition' do
+      shared_examples 'repetition' do |behavior|
+        context 'with one capture' do
+          let(:pattern) { pattern_placeholder.sub('{}', '{$_}') }
+
+          it_behaves_like behavior
+        end
+
+        context 'with two captures' do
+          let(:pattern) { pattern_placeholder.sub('{}', '{$_ | $_}') }
+
+          it_behaves_like behavior
+        end
+
+        context 'with three captures' do
+          let(:pattern) { pattern_placeholder.sub('{}', '{$_ | $_ | $_}') }
+
+          it_behaves_like behavior
+        end
+      end
+
+      context 'with ?' do
+        context 'one capture' do
+          let(:pattern_placeholder) { '(send _ _ (int {})?)' }
+
+          context 'one match' do
+            it_behaves_like 'repetition', 'single capture' do
+              let(:ruby) { 'foo(1)' }
+              let(:captured_val) { [1] }
+            end
+          end
+
+          context 'no match' do
+            it_behaves_like 'repetition', 'single capture' do
+              let(:ruby) { 'foo' }
+              let(:captured_val) { [] }
+            end
+          end
+        end
+
+        context 'two captures' do
+          let(:pattern_placeholder) { '(send _ $_ (int {})?)' }
+
+          context 'one match' do
+            it_behaves_like 'repetition', 'multiple capture' do
+              let(:ruby) { 'foo(1)' }
+              let(:captured_vals) { [:foo, [1]] }
+            end
+          end
+
+          context 'no match' do
+            it_behaves_like 'repetition', 'multiple capture' do
+              let(:ruby) { 'foo' }
+              let(:captured_vals) { [:foo, []] }
+            end
+          end
+        end
+      end
+
+      context 'with +' do
+        context 'one capture' do
+          let(:pattern_placeholder) { '(send _ _ (int {})+)' }
+
+          context 'one match' do
+            it_behaves_like 'repetition', 'single capture' do
+              let(:ruby) { 'foo(1)' }
+              let(:captured_val) { [1] }
+            end
+          end
+
+          context 'two matches' do
+            it_behaves_like 'repetition', 'single capture' do
+              let(:ruby) { 'foo(1, 2)' }
+              let(:captured_val) { [1, 2] }
+            end
+          end
+
+          context 'no match' do
+            it_behaves_like 'repetition', 'nonmatching' do
+              let(:ruby) { 'foo' }
+            end
+          end
+        end
+
+        context 'two captures' do
+          let(:pattern_placeholder) { '(send _ $_ (int {})+)' }
+
+          context 'one match' do
+            it_behaves_like 'repetition', 'multiple capture' do
+              let(:ruby) { 'foo(1)' }
+              let(:captured_vals) { [:foo, [1]] }
+            end
+          end
+
+          context 'two matches' do
+            it_behaves_like 'repetition', 'multiple capture' do
+              let(:ruby) { 'foo(1, 2)' }
+              let(:captured_vals) { [:foo, [1, 2]] }
+            end
+          end
+
+          context 'no match' do
+            it_behaves_like 'repetition', 'nonmatching' do
+              let(:ruby) { 'foo' }
+            end
+          end
+        end
+      end
+
+      context 'with *' do
+        context 'one capture' do
+          let(:pattern_placeholder) { '(send _ _ (int {})*)' }
+
+          context 'one match' do
+            it_behaves_like 'repetition', 'single capture' do
+              let(:ruby) { 'foo(1)' }
+              let(:captured_val) { [1] }
+            end
+          end
+
+          context 'two matches' do
+            it_behaves_like 'repetition', 'single capture' do
+              let(:ruby) { 'foo(1, 2)' }
+              let(:captured_val) { [1, 2] }
+            end
+          end
+
+          context 'no match' do
+            it_behaves_like 'repetition', 'single capture' do
+              let(:ruby) { 'foo' }
+              let(:captured_val) { [] }
+            end
+          end
+        end
+
+        context 'two captures' do
+          let(:pattern_placeholder) { '(send _ $_ (int {})*)' }
+
+          context 'one match' do
+            it_behaves_like 'repetition', 'multiple capture' do
+              let(:ruby) { 'foo(1)' }
+              let(:captured_vals) { [:foo, [1]] }
+            end
+          end
+
+          context 'two matches' do
+            it_behaves_like 'repetition', 'multiple capture' do
+              let(:ruby) { 'foo(1, 2)' }
+              let(:captured_vals) { [:foo, [1, 2]] }
+            end
+          end
+
+          context 'no match' do
+            it_behaves_like 'repetition', 'multiple capture' do
+              let(:ruby) { 'foo' }
+              let(:captured_vals) { [:foo, []] }
+            end
+          end
+        end
+      end
+    end
   end
 
   describe 'negation' do
@@ -1268,12 +1424,12 @@ RSpec.describe RuboCop::AST::NodePattern do
 
     context 'with an expression argument' do
       before do
-        def instance.some_function(node, arg)
+        def instance.some_function?(node, arg)
           arg === node # rubocop:disable Style/CaseEquality
         end
       end
 
-      let(:pattern) { '(send (int _value) :+ #some_function( {(int _value) (float _value)} ) )' }
+      let(:pattern) { '(send (int _value) :+ #some_function?( {(int _value) (float _value)} ) )' }
 
       context 'for which the predicate is true' do
         let(:ruby) { '2 + 2.0' }
@@ -1555,19 +1711,19 @@ RSpec.describe RuboCop::AST::NodePattern do
       module AST
         # Add test function calls
         class NodePattern
-          def goodmatch(_foo)
+          def goodmatch?(_foo)
             true
           end
 
-          def badmatch(_foo)
+          def badmatch?(_foo)
             false
           end
 
-          def witharg(foo, bar)
+          def witharg?(foo, bar)
             foo == bar
           end
 
-          def withargs(foo, bar, qux)
+          def withargs?(foo, bar, qux)
             foo.between?(bar, qux)
           end
         end
@@ -1575,14 +1731,14 @@ RSpec.describe RuboCop::AST::NodePattern do
     end
 
     context 'without extra arguments' do
-      let(:pattern) { '(lvasgn #goodmatch ...)' }
+      let(:pattern) { '(lvasgn #goodmatch? ...)' }
       let(:ruby) { 'a = 1' }
 
       it { expect(pattern).to match_code(node) }
     end
 
     context 'with one argument' do
-      let(:pattern) { '(str #witharg(%1))' }
+      let(:pattern) { '(str #witharg?(%1))' }
       let(:ruby) { '"foo"' }
       let(:params) { %w[foo] }
 
@@ -1590,7 +1746,7 @@ RSpec.describe RuboCop::AST::NodePattern do
     end
 
     context 'with multiple arguments' do
-      let(:pattern) { '(str #withargs(%1, %2))' }
+      let(:pattern) { '(str #withargs?(%1, %2))' }
       let(:ruby) { '"c"' }
       let(:params) { %w[a d] }
 
@@ -2039,11 +2195,7 @@ RSpec.describe RuboCop::AST::NodePattern do
     include RuboCop::AST::Sexp
 
     subject(:result) do
-      if keyword_params.empty? # Avoid bug in Ruby < 2.7
-        defined_class.new.send(method_name, node, *params)
-      else
-        defined_class.new.send(method_name, node, *params, **keyword_params)
-      end
+      defined_class.new.send(method_name, node, *params, **keyword_params)
     end
 
     before do
@@ -2063,6 +2215,7 @@ RSpec.describe RuboCop::AST::NodePattern do
     let(:ruby) { ':hello' }
     let(:instance) { defined_class.new }
 
+    # rubocop:disable RSpec/ExpectInLet
     let(:raise_argument_error) do
       raise_error do |err|
         expect(err).to be_a(ArgumentError)
@@ -2070,6 +2223,7 @@ RSpec.describe RuboCop::AST::NodePattern do
         expect(err.backtrace_locations.first.lineno).to be(line_no) if RUBY_ENGINE == 'ruby'
       end
     end
+    # rubocop:enable RSpec/ExpectInLet
 
     context 'with a pattern without captures' do
       let(:pattern) { '(sym _)' }
@@ -2112,7 +2266,7 @@ RSpec.describe RuboCop::AST::NodePattern do
           context 'when called on matching code' do
             it 'returns an enumerator yielding the matches' do
               is_expected.to be_a(Enumerator)
-              expect(result.to_a).to match_array [s(:sym, :hello), s(:sym, :world)]
+              expect(result.to_a).to contain_exactly(s(:sym, :hello), s(:sym, :world))
             end
           end
 
@@ -2291,14 +2445,14 @@ RSpec.describe RuboCop::AST::NodePattern do
     end
 
     context 'with a pattern with a namespaced call' do
-      let(:pattern) { '(sym #MyMod.foo)' }
+      let(:pattern) { '(sym #MyMod.foo?)' }
       let(:helper_name) { :def_node_matcher }
 
       before do
         mod = Module.new
         mod::MyMod = Module.new
         mod::MyMod.module_eval do
-          def self.foo(val)
+          def self.foo?(val)
             val == :hello
           end
         end
