@@ -1,24 +1,24 @@
 # frozen_string_literal: true
 
+require 'uri'
+
+module RuboCop
+  module AST
+    # Patch Node
+    class Node
+      # Let's make our predicate matchers read better
+      def used?
+        value_used?
+      end
+    end
+  end
+end
+
 RSpec.describe RuboCop::AST::Node do
   let(:ast) { parse_source(src).node }
   let(:node) { ast }
 
   describe '#value_used?' do
-    before :all do
-      module RuboCop # rubocop:disable Lint/ConstantDefinitionInBlock
-        module AST
-          # Patch Node
-          class Node
-            # Let's make our predicate matchers read better
-            def used?
-              value_used?
-            end
-          end
-        end
-      end
-    end
-
     context 'at the top level' do
       let(:src) { 'expr' }
 
@@ -317,7 +317,15 @@ RSpec.describe RuboCop::AST::Node do
       end
 
       context 'with no interpolation' do
-        let(:src) { URI::DEFAULT_PARSER.make_regexp.inspect }
+        let(:src) do
+          # TODO: When supporting Ruby 3.3+ runtime, `URI::DEFAULT_PARSER` can be removed and
+          #       only `URI::RFC2396_PARSER` can be used.
+          if defined?(URI::RFC2396_PARSER)
+            URI::RFC2396_PARSER
+          else
+            URI::DEFAULT_PARSER
+          end.make_regexp.inspect
+        end
 
         it 'returns true' do
           expect(node).to be_pure
@@ -423,6 +431,100 @@ RSpec.describe RuboCop::AST::Node do
         expect(node).to be_class_constructor
       end
     end
+
+    context 'using Ruby >= 2.7', :ruby27 do
+      context 'class definition with a numblock' do
+        let(:src) { 'Class.new { do_something(_1) }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+
+      context 'module definition with a numblock' do
+        let(:src) { 'Module.new { do_something(_1) }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+
+      context 'Struct definition with a numblock' do
+        let(:src) { 'Struct.new(:foo, :bar) { do_something(_1) }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+    end
+
+    context 'using Ruby >= 3.2', :ruby32 do
+      context 'Data definition with a block' do
+        let(:src) { 'Data.define(:foo, :bar) { def a = 42 }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+
+      context 'Data definition with a numblock' do
+        let(:src) { 'Data.define(:foo, :bar) { do_something(_1) }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+
+      context 'Data definition without block' do
+        let(:src) { 'Data.define(:foo, :bar)' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+
+      context '::Data' do
+        let(:src) { '::Data.define(:foo, :bar) { def a = 42 }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+    end
+
+    context 'using Ruby >= 3.4', :ruby34 do
+      context 'class definition with an itblock' do
+        let(:src) { 'Class.new { do_something(it) }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+
+      context 'module definition with an itblock' do
+        let(:src) { 'Module.new { do_something(it) }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+
+      context 'Struct definition with an itblock' do
+        let(:src) { 'Struct.new(:foo, :bar) { do_something(it) }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+
+      context 'Data definition with an itblock' do
+        let(:src) { 'Data.define(:foo, :bar) { do_something(it) }' }
+
+        it 'matches' do
+          expect(node).to be_class_constructor
+        end
+      end
+    end
   end
 
   describe '#struct_constructor?' do
@@ -831,7 +933,7 @@ RSpec.describe RuboCop::AST::Node do
     context 'when string literal' do
       let(:src) { '"42"' }
 
-      it 'is true' do
+      it 'is false' do
         expect(node).not_to be_numeric_type
       end
     end
@@ -918,4 +1020,352 @@ RSpec.describe RuboCop::AST::Node do
       end
     end
   end
+
+  describe '#const_name' do
+    context 'when given a `const` node' do
+      context 'FOO' do
+        let(:src) { 'FOO' }
+
+        it 'returns FOO' do
+          expect(node.const_name).to eq('FOO')
+        end
+      end
+
+      context 'FOO::BAR::BAZ' do
+        let(:src) { 'FOO::BAR::BAZ' }
+
+        it 'returns FOO::BAR::BAZ' do
+          expect(node.const_name).to eq('FOO::BAR::BAZ')
+        end
+      end
+
+      context '::FOO::BAR::BAZ' do
+        let(:src) { '::FOO::BAR::BAZ' }
+
+        it 'returns FOO::BAR::BAZ' do
+          expect(node.const_name).to eq('FOO::BAR::BAZ')
+        end
+      end
+    end
+
+    context 'when given a `casgn` node' do
+      context 'FOO = 1' do
+        let(:src) { 'FOO = 1' }
+
+        it 'returns FOO' do
+          expect(node.const_name).to eq('FOO')
+        end
+      end
+
+      context 'FOO::BAR::BAZ = 1' do
+        let(:src) { 'FOO::BAR::BAZ = 1' }
+
+        it 'returns FOO::BAR::BAZ' do
+          expect(node.const_name).to eq('FOO::BAR::BAZ')
+        end
+      end
+
+      context '::FOO::BAR::BAZ = 1' do
+        let(:src) { '::FOO::BAR::BAZ = 1' }
+
+        it 'returns FOO::BAR::BAZ' do
+          expect(node.const_name).to eq('FOO::BAR::BAZ')
+        end
+      end
+    end
+
+    context 'when given a `send` node' do
+      let(:src) { 'foo.bar' }
+
+      it 'return nil' do
+        expect(node.const_name).to be_nil
+      end
+    end
+  end
+
+  describe '#any_match_pattern_type?' do
+    # Ruby 2.7's one-line `in` pattern node type is `match-pattern`.
+    context 'when `in` one-line pattern matching', :ruby27 do
+      let(:src) { 'expression in pattern' }
+
+      it 'is true' do
+        if node.in_match_type?
+          skip "`in_match` from `AST::Builder.modernize` isn't treated as one-line pattern matching."
+        end
+
+        expect(node).to be_any_match_pattern_type
+      end
+    end
+
+    # Ruby 3.0's one-line `in` pattern node type is `match-pattern-p`.
+    context 'when `in` one-line pattern matching', :ruby30 do
+      let(:src) { 'expression in pattern' }
+
+      it 'is true' do
+        expect(node).to be_any_match_pattern_type
+      end
+    end
+
+    # Ruby 3.0's one-line `=>` pattern node type is `match-pattern`.
+    context 'when `=>` one-line pattern matching', :ruby30 do
+      let(:src) { 'expression => pattern' }
+
+      it 'is true' do
+        expect(node).to be_any_match_pattern_type
+      end
+    end
+
+    context 'when pattern matching', :ruby27 do
+      let(:src) do
+        <<~RUBY
+          case expression
+          in pattern
+          end
+        RUBY
+      end
+
+      it 'is false' do
+        expect(node).not_to be_any_match_pattern_type
+      end
+    end
+  end
+
+  describe '#any_str_type?' do
+    context 'when string literal' do
+      let(:src) { "'foo'" }
+
+      it 'is true' do
+        expect(node).to be_any_str_type
+      end
+    end
+
+    context 'when interpolated string' do
+      let(:src) { %q("foo #{bar}") }
+
+      it 'is true' do
+        expect(node).to be_any_str_type
+      end
+    end
+
+    context 'when `xstr` node' do
+      let(:src) { '`ls`' }
+
+      it 'is true' do
+        expect(node).to be_any_str_type
+      end
+    end
+
+    context 'when numeric literal' do
+      let(:src) { '42' }
+
+      it 'is false' do
+        expect(node).not_to be_any_str_type
+      end
+    end
+  end
+
+  describe '#any_sym_type?' do
+    context 'when symbol' do
+      let(:src) { ':str' }
+
+      it 'is true' do
+        expect(node).to be_any_sym_type
+      end
+    end
+
+    context 'when interpolated symbol' do
+      let(:src) { ':"#{foo}"' }
+
+      it 'is true' do
+        expect(node).to be_any_sym_type
+      end
+    end
+
+    context 'when string' do
+      let(:src) { "'foo'" }
+
+      it 'is false' do
+        expect(node).not_to be_any_sym_type
+      end
+    end
+
+    context 'when interpolated string' do
+      let(:src) { %q("foo #{bar}") }
+
+      it 'is false' do
+        expect(node).not_to be_any_sym_type
+      end
+    end
+
+    context 'when `xstr` node' do
+      let(:src) { '`ls`' }
+
+      it 'is false' do
+        expect(node).not_to be_any_sym_type
+      end
+    end
+
+    context 'when numeric literal' do
+      let(:src) { '42' }
+
+      it 'is false' do
+        expect(node).not_to be_any_sym_type
+      end
+    end
+  end
+
+  describe '#type?' do
+    let(:src) do
+      <<~RUBY
+        foo.bar
+      RUBY
+    end
+
+    context 'when it is one of the given types' do
+      it 'is true' do
+        expect(node).to be_type(:send, :const, :lvar)
+      end
+    end
+
+    context 'when it is not one of the given types' do
+      it 'is false' do
+        expect(node).not_to be_type(:if, :while, :until)
+      end
+    end
+
+    context 'when given :argument with an `arg` node' do
+      let(:src) { 'foo { |>> var <<| } ' }
+
+      it 'is true' do
+        arg_node = ast.procarg0_type? ? ast.child_nodes.first : node
+        expect(arg_node).to be_type(:argument)
+      end
+    end
+
+    context 'when given :boolean with an `true` node' do
+      let(:src) { 'true' }
+
+      it 'is true' do
+        expect(node).to be_type(:boolean)
+      end
+    end
+
+    context 'when given :numeric with an `int` node' do
+      let(:src) { '42' }
+
+      it 'is true' do
+        expect(node).to be_type(:numeric)
+      end
+    end
+
+    context 'when given :range with an `irange` node' do
+      let(:src) { '1..3' }
+
+      it 'is true' do
+        expect(node).to be_type(:range)
+      end
+    end
+
+    context 'when given :call with an `send` node' do
+      let(:src) { 'foo.bar' }
+
+      it 'is true' do
+        expect(node).to be_type(:call)
+      end
+    end
+  end
+
+  describe '#loc?' do
+    let(:src) { '%i[>> sym << sym2]' }
+
+    before(:all) { RSpec::Matchers.alias_matcher :have_loc, :be_loc }
+
+    context 'when loc exists' do
+      let(:src) { ':sym' }
+
+      it 'returns true when the location exists' do
+        expect(node).to have_loc(:begin)
+      end
+    end
+
+    it 'returns false when requested loc is `nil`' do
+      expect(node).not_to have_loc(:begin)
+    end
+
+    it 'returns false when requested loc does not exist' do
+      expect(node).not_to have_loc(:foo)
+    end
+  end
+
+  describe '#loc_is?' do
+    let(:src) { '%i[>> sym << sym2]' }
+
+    context 'when loc exists' do
+      let(:src) { ':sym' }
+
+      it 'returns true when loc matches argument' do
+        expect(node).to be_loc_is(:begin, ':')
+      end
+
+      it 'returns false when loc does not match argument' do
+        expect(node).not_to be_loc_is(:begin, '!')
+      end
+    end
+
+    it 'returns false when requested loc is `nil`' do
+      expect(node).not_to be_loc_is(:begin, ':')
+    end
+
+    it 'returns false when requested loc does not exist' do
+      expect(node).not_to be_loc_is(:foo, ':')
+    end
+  end
+
+  context 'traversal' do
+    shared_examples 'traverse' do |test_name, *types, result|
+      it "handles #{test_name}" do
+        expect(node.send(method, *types).map(&:type)).to eq(result)
+      end
+    end
+
+    describe '#each_ancestor' do
+      let(:src) { 'foo&.bar([>>baz<<]).bat' }
+      let(:method) { :each_ancestor }
+
+      it_behaves_like 'traverse', 'no argument', %i[array csend send]
+      it_behaves_like 'traverse', 'single argument', :send, %i[send]
+      it_behaves_like 'traverse', 'two arguments', :send, :csend, %i[csend send]
+      it_behaves_like 'traverse', 'group argument', :call, %i[csend send]
+    end
+
+    describe '#each_child_node' do
+      let(:src) { '[1, 2.0, foo, 3i, 4, BAR, 5r]' }
+      let(:method) { :each_child_node }
+
+      it_behaves_like 'traverse', 'no argument', %i[int float send complex int const rational]
+      it_behaves_like 'traverse', 'single argument', :int, %i[int int]
+      it_behaves_like 'traverse', 'two arguments', :int, :complex, :csend, %i[int complex int]
+      it_behaves_like 'traverse', 'group argument', :numeric, %i[int float complex int rational]
+    end
+
+    describe '#each_descendant' do
+      let(:src) { 'foo(true, false, bar(baz, true))' }
+      let(:method) { :each_descendant }
+
+      it_behaves_like 'traverse', 'no argument', %i[true false send send true]
+      it_behaves_like 'traverse', 'single argument', :true, %i[true true]
+      it_behaves_like 'traverse', 'two arguments', :false, :send, %i[false send send]
+      it_behaves_like 'traverse', 'group argument', :boolean, %i[true false true]
+    end
+
+    describe '#each_node' do
+      let(:src) { 'def foo(bar, *, baz, **kw, &block); 123; end' }
+      let(:method) { :each_node }
+
+      it_behaves_like 'traverse', 'no argument', %i[def args arg restarg arg kwrestarg blockarg int]
+      it_behaves_like 'traverse', 'single argument', :arg, %i[arg arg]
+      it_behaves_like 'traverse', 'two arguments', :restarg, :blockarg, %i[restarg blockarg]
+      it_behaves_like 'traverse', 'group argument', :argument, %i[arg restarg arg kwrestarg blockarg]
+    end
+  end
 end
