@@ -1,7 +1,11 @@
 # frozen_string_literal: true
 
 RSpec.describe RuboCop::AST::ProcessedSource do
-  subject(:processed_source) { described_class.new(source, ruby_version, path) }
+  subject(:processed_source) do
+    described_class.new(
+      source, ruby_version, path, parser_engine: parser_engine, prism_result: prism_result
+    )
+  end
 
   let(:source) { <<~RUBY }
     # an awesome method
@@ -10,6 +14,9 @@ RSpec.describe RuboCop::AST::ProcessedSource do
     end
     some_method
   RUBY
+  let(:prism_result) { nil }
+  let(:prism_parse_lex_result) { Prism.parse_lex(source) }
+
   let(:ast) { processed_source.ast }
   let(:path) { 'ast/and_node_spec.rb' }
 
@@ -17,6 +24,102 @@ RSpec.describe RuboCop::AST::ProcessedSource do
     let(:source) { "# \xf9" }
   end
 
+  describe '#initialize' do
+    context 'when parsing non UTF-8 frozen string' do
+      let(:source) { (+'true').force_encoding(Encoding::ASCII_8BIT).freeze }
+
+      it 'returns an instance of ProcessedSource' do
+        is_expected.to be_a(described_class)
+      end
+    end
+
+    context 'when parsing code with an invalid encoding comment' do
+      let(:source) { '# encoding: foobar' }
+
+      it 'returns a parser error' do
+        expect(processed_source.parser_error).to be_a(Parser::UnknownEncodingInMagicComment)
+        expect(processed_source.parser_error.message)
+          .to include('unknown encoding name - foobar')
+      end
+    end
+
+    shared_examples 'invalid parser_engine' do
+      it 'raises ArgumentError' do
+        expect { processed_source }.to raise_error(ArgumentError) do |e|
+          expected = 'The keyword argument `parser_engine` accepts .*, ' \
+                     "but `#{parser_engine}` was passed."
+          expect(e.message).to match(expected)
+        end
+      end
+    end
+
+    context 'when using an invalid `parser_engine` symbol argument' do
+      let(:parser_engine) { :unknown_parser_engine }
+
+      it_behaves_like 'invalid parser_engine'
+    end
+
+    context 'when using an invalid `parser_engine` string argument' do
+      let(:parser_engine) { 'unknown_parser_engine' }
+
+      it_behaves_like 'invalid parser_engine'
+    end
+
+    context 'when using `parser_engine: :parser_prism` and `prism_result` with a `ParseLexResult`' do
+      let(:ruby_version) { 3.4 }
+      let(:parser_engine) { :parser_prism }
+      let(:prism_result) { prism_parse_lex_result }
+
+      it 'returns an instance of ProcessedSource' do
+        is_expected.to be_a(described_class)
+      end
+    end
+
+    context 'when `parser_engine` is `parser_whitequark`' do
+      let(:parser_engine) { :parser_whitequark }
+
+      context 'and Ruby 3.4 is requested' do
+        let(:ruby_version) { 3.4 }
+
+        it 'uses `parser_whitequark`' do
+          expect(processed_source.parser_engine).to eq(:parser_whitequark)
+        end
+      end
+    end
+
+    context 'when `parser_engine` is `parser_prism`' do
+      let(:parser_engine) { :parser_prism }
+
+      context 'and Ruby 3.3 is requested' do
+        let(:ruby_version) { 3.3 }
+
+        it 'uses `parser_prism`' do
+          expect(processed_source.parser_engine).to eq(:parser_prism)
+        end
+      end
+    end
+
+    context 'when `parser_engine` is `:default`' do
+      let(:parser_engine) { :default }
+
+      context 'and Ruby 3.3 is requested' do
+        let(:ruby_version) { 3.3 }
+
+        it 'uses `parser_whitequark`' do
+          expect(processed_source.parser_engine).to eq(:parser_whitequark)
+        end
+      end
+
+      context 'and Ruby 3.4 is requested' do
+        let(:ruby_version) { 3.4 }
+
+        it 'uses `parser_prism`' do
+          expect(processed_source.parser_engine).to eq(:parser_prism)
+        end
+      end
+    end
+  end
+
   describe '.from_file' do
     describe 'when the file exists' do
       around do |example|
@@ -26,7 +129,9 @@ RSpec.describe RuboCop::AST::ProcessedSource do
         Dir.chdir(org_pwd)
       end
 
-      let(:processed_source) { described_class.from_file(path, ruby_version) }
+      let(:processed_source) do
+        described_class.from_file(path, ruby_version, parser_engine: parser_engine)
+      end
 
       it 'returns an instance of ProcessedSource' do
         is_expected.to be_a(described_class)
@@ -48,18 +153,48 @@ RSpec.describe RuboCop::AST::ProcessedSource do
     it 'is the path passed to .new' do
       expect(processed_source.path).to eq(path)
     end
+
+    context 'when using `parser_engine: :parser_prism` and `prism_result` with a `ParseLexResult`' do
+      let(:ruby_version) { 3.4 }
+      let(:parser_engine) { :parser_prism }
+      let(:prism_result) { prism_parse_lex_result }
+
+      it 'is the path passed to .new' do
+        expect(processed_source.path).to eq(path)
+      end
+    end
   end
 
   describe '#buffer' do
     it 'is a source buffer' do
       expect(processed_source.buffer).to be_a(Parser::Source::Buffer)
     end
+
+    context 'when using `parser_engine: :parser_prism` and `prism_result` with a `ParseLexResult`' do
+      let(:ruby_version) { 3.4 }
+      let(:parser_engine) { :parser_prism }
+      let(:prism_result) { prism_parse_lex_result }
+
+      it 'is a source buffer' do
+        expect(processed_source.buffer).to be_a(Parser::Source::Buffer)
+      end
+    end
   end
 
   describe '#ast' do
     it 'is the root node of AST' do
       expect(processed_source.ast).to be_a(RuboCop::AST::Node)
     end
+
+    context 'when using `parser_engine: :parser_prism` and `prism_result` with a `ParseLexResult`' do
+      let(:ruby_version) { 3.4 }
+      let(:parser_engine) { :parser_prism }
+      let(:prism_result) { prism_parse_lex_result }
+
+      it 'is the root node of AST' do
+        expect(processed_source.ast).to be_a(RuboCop::AST::Node)
+      end
+    end
   end
 
   describe '#comments' do
@@ -297,7 +432,7 @@ RSpec.describe RuboCop::AST::ProcessedSource do
       let(:hash) { array.children[1] }
 
       context 'provided source_range on line without comment' do
-        let(:range) { hash.pairs.first.loc.expression }
+        let(:range) { hash.pairs.first.source_range }
 
         it { is_expected.to be false }
       end
@@ -309,13 +444,13 @@ RSpec.describe RuboCop::AST::ProcessedSource do
       end
 
       context 'provided source_range on line with comment' do
-        let(:range) { hash.pairs.last.loc.expression }
+        let(:range) { hash.pairs.last.source_range }
 
         it { is_expected.to be true }
       end
 
       context 'provided a multiline source_range with at least one line with comment' do
-        let(:range) { array.loc.expression }
+        let(:range) { array.source_range }
 
         it { is_expected.to be true }
       end
@@ -400,6 +535,7 @@ RSpec.describe RuboCop::AST::ProcessedSource do
     end
   end
 
+  # rubocop:disable RSpec/RedundantPredicateMatcher
   describe '#start_with?' do
     context 'with blank source' do
       let(:source) { '' }
@@ -429,6 +565,7 @@ RSpec.describe RuboCop::AST::ProcessedSource do
       end
     end
   end
+  # rubocop:enable RSpec/RedundantPredicateMatcher
 
   describe '#preceding_line' do
     let(:source) { <<~RUBY }
