@@ -118,6 +118,18 @@ RSpec.describe RuboCop::AST::SendNode do
       it { expect(send_node).to be_access_modifier }
     end
 
+    context 'when node is a non-bare `module_function` with multiple arguments' do
+      let(:source) do
+        <<~RUBY
+          module Foo
+          >> module_function :foo, :bar <<
+          end
+        RUBY
+      end
+
+      it { expect(send_node).to be_access_modifier }
+    end
+
     context 'when node is not an access modifier' do
       let(:source) do
         <<~RUBY
@@ -194,6 +206,21 @@ RSpec.describe RuboCop::AST::SendNode do
         it { expect(send_node).to be_bare_access_modifier }
       end
     end
+
+    context 'with Ruby >= 3.4', :ruby34 do
+      context 'when node is access modifier in itblock' do
+        let(:source) do
+          <<~RUBY
+            included do
+            it
+            >> module_function <<
+            end
+          RUBY
+        end
+
+        it { expect(send_node).to be_bare_access_modifier }
+      end
+    end
   end
 
   describe '#non_bare_access_modifier?' do
@@ -209,6 +236,18 @@ RSpec.describe RuboCop::AST::SendNode do
       it { expect(send_node).to be_non_bare_access_modifier }
     end
 
+    context 'when node is a non-bare `module_function` with multiple arguments' do
+      let(:source) do
+        <<~RUBY
+          module Foo
+          >> module_function :foo, :bar <<
+          end
+        RUBY
+      end
+
+      it { expect(send_node).to be_non_bare_access_modifier }
+    end
+
     context 'when node does not have an argument' do
       let(:source) do
         <<~RUBY
@@ -315,6 +354,19 @@ RSpec.describe RuboCop::AST::SendNode do
         end
       end
 
+      context 'with Ruby >= 3.4', :ruby27 do
+        context 'when parent is an itblock in a macro scope' do
+          let(:source) do
+            ['concern :Auth do',
+             '>>bar :baz<<',
+             '  bar it',
+             'end'].join("\n")
+          end
+
+          it { expect(send_node).to be_macro }
+        end
+      end
+
       context 'when parent is a block not in a macro scope' do
         let(:source) { <<~RUBY }
           class Foo
@@ -1082,6 +1134,14 @@ RSpec.describe RuboCop::AST::SendNode do
         it { expect(send_node).to be_block_literal }
       end
     end
+
+    context 'with Ruby >= 3.4', :ruby27 do
+      context 'with an itblock literal' do
+        let(:source) { '>> foo.bar << { baz(it) }' }
+
+        it { expect(send_node).to be_block_literal }
+      end
+    end
   end
 
   describe '#arithmetic_operation?' do
@@ -1130,6 +1190,14 @@ RSpec.describe RuboCop::AST::SendNode do
         it { expect(send_node.block_node).to be_numblock_type }
       end
     end
+
+    context 'with Ruby >= 3.4', :ruby34, broken_on: :parser do
+      context 'with an itblock literal' do
+        let(:source) { '>>foo.bar<< { baz(it) }' }
+
+        it { expect(send_node.block_node).to be_itblock_type }
+      end
+    end
   end
 
   describe '#splat_argument?' do
