@@ -319,6 +319,7 @@ RSpec.describe RuboCop::AST::Token do
       let(:source) { <<~RUBY }
         { a: 1 }
         foo { |f| bar(f) }
+        -> { f }
       RUBY
 
       let(:left_hash_brace_token) do
@@ -331,6 +332,9 @@ RSpec.describe RuboCop::AST::Token do
       let(:left_block_brace_token) do
         processed_source.find_token { |t| t.text == '{' && t.line == 2 }
       end
+      let(:left_lambda_brace_token) do
+        processed_source.find_token { |t| t.text == '{' && t.line == 3 }
+      end
       let(:left_parens_token) do
         processed_source.find_token { |t| t.text == '(' }
       end
@@ -355,6 +359,7 @@ RSpec.describe RuboCop::AST::Token do
       describe '#left_curly_brace?' do
         it 'returns true for left block brace tokens' do
           expect(left_block_brace_token).to be_left_curly_brace
+          expect(left_lambda_brace_token).to be_left_curly_brace
         end
 
         it 'returns false for non left block brace tokens' do
